require 'logger'

module Coralogix
    # @private
    class DebugLogger

        # attr_accessor :external, :enabled, :logger

        def self.initialize
            begin
                @mutex = Mutex.new
                @enabled = false
                @external = false
            rescue Exception => e
                if @enabled 
                    puts e.message  
                    puts e.backtrace.inspect  
                end
            end
        end 

        def self.enabled=(value)
            @enabled = value
        end

        def self.external=(value)
            @external = value
        end

        def self.logger=(value)
            @logger = value
        end

        def self.debug_mode?
            @enabled
        end

        def self.debug_mode=(value)
            begin
                @enabled = value
                if value
                    @logger = Logger.new("/tmp/#{LOG_FILE_NAME}", 1, 10485760)
                else
                    if @external == false
                        @logger.close unless @logger == nil
                        @logger = nil
                    end
                end
            rescue Exception => e
                if @enabled  
                    puts e.message  
                    puts e.backtrace.inspect  
                end
            end
        end

        Logger::Severity.const_set("TRACE", 0)
        Logger::Severity.constants.each do |level|
            define_singleton_method("#{level.downcase}") do |*args|
                if @enabled
                    @mutex.synchronize do
                        begin
                            if @external == false
                                puts "#{__method__.upcase}:  #{Time.now.strftime('%H:%M:%S.%L')} - #{args}"
                            end
                            @logger.send("#{__method__}", args)
                        rescue Exception => e
                            puts e.message  
                            puts e.backtrace.inspect  
                        end
                    end
                end
            end
        end

        initialize
    
    end

end